@@ -7,16 +7,13 @@ import argparse
 import random
 import time
 from datetime import datetime
+from dotenv import load_dotenv
+from atop.modules.telegramhelper import TelegramHelper
+from atop.modules.const import user_agent
+import os
 
 delays = [0.2, 0.5, 0.6, 0.5, 0.1, 0.4, 1]
 
-uapools = [
-    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2224.3 Safari/537.36",
-    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36",
-    "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:53.0) Gecko/20100101 Firefox/53.0",
-    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9) Gecko/20080705 Firefox/3.0 Kapiko/3.0",
-    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.79",
-]
 
 def gdelay():
     return random.choice(delays)
@@ -49,19 +46,19 @@ Welcome in the realm of....."""
 ░░         ░░   ░  ▒ ░     ░░    ░   ▒   ░       ▒ ▒ ░░          
             ░      ░        ░        ░  ░░ ░     ░ ░             
                            ░             ░       ░ ░             
-v 0.0.2 """
+v 0.1.8 """
         + Style.RESET_ALL
     )
 
 
 class Ton_retriever:
-
+    telegram_pivot = False
     silent = False
     step = 10000
     offset = 0
     target = ""
     tor_proxy = False
-    # comprensive scan use Telethon api to retrieve info about users
+
     comprehensive = False
     currentua = ""
     address = ""
@@ -76,6 +73,16 @@ class Ton_retriever:
     ens_detail = None
     session = None
 
+    asset_in_sale = False
+
+    # pivoting Telegram account
+    api_hash = None
+    api_id = None
+    api_telephone = None
+    sessionstring = None
+    tgchecker = None
+    outputdir = None
+
     proxy = False
     s_proto = "socks5"
     s_proxy = "127.0.0.1"
@@ -83,7 +90,7 @@ class Ton_retriever:
 
     @staticmethod
     def sleeping_time():
-        time.sleep(gdelay(delays))
+        time.sleep(gdelay())
 
     @staticmethod
     def ipf_ens(domain, session=None):
@@ -112,7 +119,55 @@ class Ton_retriever:
             self.user_agent_retrieve(self)
             self.session.headers = {"User-Agent": self.ua()}
 
-    def __init__(self, _telephone_num, _comprehensive, _tor, _silent):
+    def __init__(
+        self,
+        _telephone_num,
+        _comprehensive,
+        _tor,
+        _silent,
+        _telegram_pivot,
+        outputdir=None,
+        telegram_api_hash=None,
+        telegram_api_id=None,
+        telegram_api_telephone=None,
+        sessionstring=None,
+    ):
+        if _telegram_pivot:
+            load_dotenv()
+            try:
+                self.api_id = os.getenv("API_ID")
+                self.api_hash = os.getenv("API_HASH")
+                self.api_telephone = os.getenv("PHONE_NUMBER")
+                self.sessionstring = os.getenv("SESSION_STRING")
+                if not self.api_id and telegram_api_id:
+                    self.api_id = telegram_api_id
+                if not self.api_hash and telegram_api_hash:
+                    self.api_hash = telegram_api_hash
+                if not self.api_telephone and telegram_api_telephone:
+                    self.api_telephone = telegram_api_telephone
+                if not self.sessionstring and sessionstring:
+                    self.sessionstring = sessionstring
+                self.outputdir = outputdir
+                if not (
+                    (self.api_telephone != "" or self.sessionstring)
+                    and (self.api_hash != "" and self.api_telephone != "")
+                ):
+                    print(
+                        "You have to setup your sockpuppet.. you haven't compiled your env data.. "
+                    )
+                    exit(0)
+                else:
+                    self.telegram_pivot = True
+                self.tgchecker = TelegramHelper(
+                    self.api_hash,
+                    self.api_id,
+                    self.api_telephone,
+                    self.outputdir,
+                    self.sessionstring,
+                )
+            except:
+                pass
+
         self.comprehensive = _comprehensive
         if not self.check_format(_telephone_num):
             if not self.silent:
@@ -173,7 +228,7 @@ class Ton_retriever:
                         if count == 0:
                             break
         except Exception as exx:
-            self.user_agents = uapools
+            self.user_agents = user_agent
 
     def ua(self):
         return random.choice(self.user_agents)
@@ -183,16 +238,18 @@ class Ton_retriever:
             print(f" [-] {self.kind} NOT FOUND, {self.offset} {self.kind} PROCESSED...")
             return
         else:
-
             balance = "N/A"
             last_date = datetime.min
-            if self.comprehensive:
-                print(
-                    """
+            print(
+                """
       ░▒████████████████████ TON ██████████████████████▒░                   
                     """
-                )
-            print(" [+] ", f"Details for {self.kind.lower()}: " + self.target)
+            )
+
+            header = f" [+] Details for {self.kind.lower()}: " + self.target
+            if self.asset_in_sale:
+                header = header + " ( asset in sale! )"
+            print(header)
             print("  ├  Owner address: ", str(self.address))
             print("  ├  Is scam: ", str(self.is_scam))
             if self.owner_name != "":
@@ -219,7 +276,6 @@ class Ton_retriever:
                             % len(self.nfts["data"]["nftItemsByOwner"]["items"]),
                         )
                         processnft = True
-
             if processnft:
                 first = True
                 for nftff in self.nfts["data"]["nftItemsByOwner"]["items"]:
@@ -227,10 +283,24 @@ class Ton_retriever:
                         print("  |")
                     print("  ├  Address: %s" % (nftff["address"]))
                     print("  |  Name: %s, Kind: %s" % (nftff["name"], nftff["kind"]))
-                    if "collection" in nftff.keys():
-                        print("  |  Collection: %s" % (nftff["collection"]["name"]))
-                    if "image" in nftff.keys():
-                        if "originalUrl" in nftff["image"].keys():
+                    is_collection = ""
+                    if "collection" in nftff.keys() and nftff["collection"]:
+                        is_collection = nftff["collection"]["name"]
+                        print("  |  Collection: %s" % (is_collection))
+                        ## pivoting Telegram account
+                        if (
+                            "tg-data" in nftff.keys()
+                            and self.telegram_pivot
+                            and nftff["tg-data"]
+                        ):
+                            if nftff["tg-data"][1] and nftff["tg-data"][1] > 0:
+                                TelegramHelper.print_entity(nftff["tg-data"])
+
+                    if "image" in nftff.keys() and nftff["image"]:
+                        if (
+                            "originalUrl" in nftff["image"].keys()
+                            and nftff["image"]["originalUrl"]
+                        ):
                             print("  |  Url: %s" % (nftff["image"]["originalUrl"]))
                     first = False
                 print("  └  ------------------------------------")
@@ -255,17 +325,20 @@ class Ton_retriever:
                             )
                             date = datetime.fromtimestamp(
                                 int(
-                                    self.ens_detail["data"]["domains"][0]["registration"][
-                                        "registrationDate"
-                                    ]
+                                    self.ens_detail["data"]["domains"][0][
+                                        "registration"
+                                    ]["registrationDate"]
                                 )
                             )
-                            print("  ├  Registration: ", date.strftime("%Y-%m-%d %H:%M:%S"))
+                            print(
+                                "  ├  Registration: ",
+                                date.strftime("%Y-%m-%d %H:%M:%S"),
+                            )
                             date = datetime.fromtimestamp(
                                 int(
-                                    self.ens_detail["data"]["domains"][0]["registration"][
-                                        "expiryDate"
-                                    ]
+                                    self.ens_detail["data"]["domains"][0][
+                                        "registration"
+                                    ]["expiryDate"]
                                 )
                             )
                             print("  ├  Expiry: ", date.strftime("%Y-%m-%d %H:%M:%S"))
@@ -294,6 +367,48 @@ class Ton_retriever:
                     first = False
                 print("  └  ------------------------------------")
 
+    def enrich_telegram_asset(self):
+        try:
+            for currentnft in self.nfts["data"]["nftItemsByOwner"]["items"]:
+                currentnft["tg-data"] = None
+                if "collection" in currentnft.keys() and currentnft["collection"]:
+                    if (
+                        currentnft["collection"]["name"].strip()
+                        == "Anonymous Telegram Numbers"
+                        and self.telegram_pivot
+                    ):
+                        if self.tgchecker:
+                            currentnft[
+                                "tg-data"
+                            ] = self.tgchecker.check_telegram_number(currentnft["name"])
+                    if (
+                        currentnft["collection"]["name"].strip() == "Telegram Usernames"
+                        and self.telegram_pivot
+                    ):
+                        if self.tgchecker:
+                            nickname_data = self.tgchecker.check_telegram_nickname(
+                                currentnft["name"]
+                            )
+                            if nickname_data:
+                                if nickname_data["apidetail"]:
+                                    currentnft["tg-data"] = [
+                                        currentnft["name"],
+                                        nickname_data["apidetail"].id,
+                                        nickname_data,
+                                    ]
+                                else:
+                                    currentnft["tg-data"] = [
+                                        currentnft["name"],
+                                        -1,
+                                        None,
+                                    ]
+                            else:
+                                currentnft["tg-data"] = [currentnft["name"], -1, None]
+        except Exception as exx:
+            if not self.silent:
+                print(f"[-] AN ISSUE OCCURRED DURING RETRIEVING TELEGRAM INFO... {exx}")
+            exit(1)
+
     def pivot_ens(self):
         ens_domain = self.target.split(".")[0] + ".eth"
         request_api = "https://api.thegraph.com/subgraphs/name/ensdomains/ens"
@@ -307,7 +422,7 @@ class Ton_retriever:
             self.ens_detail = json.loads(res)
         except Exception as exx:
             if not self.silent:
-                print("[-] AN ISSUE OCCURRED DURING RETRIEVING ENS INFO...")
+                print(f"[-] AN ISSUE OCCURRED DURING RETRIEVING ENS INFO... {exx}")
             exit(1)
 
     def request_address_nft(self, addr):
@@ -319,6 +434,8 @@ class Ton_retriever:
         try:
             res = self.session.post(request_api, json=req_body).text
             self.nfts = json.loads(res)
+            if self.tgchecker:
+                self.enrich_telegram_asset()
         except Exception as exx:
             if not self.silent:
                 print("[-] AN ISSUE OCCURRED DURING RETRIEVING NFT INFO...")
@@ -362,7 +479,6 @@ class Ton_retriever:
                     exit(1)
 
             for element in obj["nft_items"]:
-
                 count += 1
                 search_field = ""
                 if "name" in element["metadata"].keys():
@@ -377,14 +493,28 @@ class Ton_retriever:
 
                 if search_field == self.target:
                     self.nft_name = search_field
-                    self.address = element["owner"]["address"]
-                    self.is_scam = element["owner"]["is_scam"]
-                    if "name" in element["owner"].keys():
+                    _owner = element["owner"]["address"]
+                    _is_scam = element["owner"]["is_scam"]
+
+                    # handling auction smartcontract
+                    if (
+                        "sale" in element.keys()
+                        and element["owner"]["address"]
+                        != element["sale"]["owner"]["address"]
+                    ):
+                        self.asset_in_sale = True
+                        _owner = element["sale"]["owner"]["address"]
+                        _is_scam = element["sale"]["owner"]["is_scam"]
+                        if "name" in element["sale"]["owner"]:
+                            self.owner_name = element["sale"]["owner"]["name"]
+                    elif "name" in element["owner"].keys():
                         self.owner_name = element["owner"]["name"]
 
-                    self.request_address_info(element["owner"]["address"])
-                    self.request_address_transctions(element["owner"]["address"])
-                    self.request_address_nft(element["owner"]["address"])
+                    self.address = _owner
+                    self.is_scam = _is_scam
+                    self.request_address_info(_owner)
+                    self.request_address_transctions(_owner)
+                    self.request_address_nft(_owner)
                     self.stop_cycle = True
                     break
 
@@ -407,19 +537,27 @@ class Ton_retriever:
             if self.kind == "DOMAIN":
                 self.pivot_ens()
 
-
+    @staticmethod
+    def telegram_generate_session():
+        TelegramHelper.generate_string_session()
 
 
 def run():
     parser = argparse.ArgumentParser(
-        description="Launch me, and u'll receive a TON of privacy..."
+        description="Launch me, and you'll receive a TON of privacy..."
     )
     parser.add_argument(
         "--target",
-        required=True,
+        required=False,
         help=" [?] TON number, nickname or domain to analyze ...",
     )
-
+    parser.add_argument(
+        "-l",
+        "--login",
+        default=False,
+        help=" [?] Create session string for Telegram login ...",
+        action="store_true",
+    )
     """
     if a flag comprehensive is True
     a deep inspection will be done:
@@ -432,7 +570,7 @@ def run():
         "--comprehensive",
         required=False,
         default=False,
-        help=" [?] Exhaustive research [work in progress] ...",
+        help=" [?] Comprehensive research which includes pivoting on ENS domains ...",
         action="store_true",
     )
     parser.add_argument(
@@ -443,34 +581,50 @@ def run():
         help=" [?] Use TOR as SOCK5 proxy ...",
         action="store_true",
     )
+    parser.add_argument(
+        "-p",
+        "--phonepivot",
+        required=False,
+        default=False,
+        help=" [?] Pivot Telegram account ...",
+        action="store_true",
+    )
     parser.add_argument(
         "-s",
         "--silent",
         required=False,
         default=False,
-        help=" [?] Use TOR as SOCK5 proxy ...",
+        help=" [?] Disable any print, useful if you don't want to print on stdout ...",
         action="store_true",
     )
-    """
-    mandatory if you want to use Telepathy ??
-    """
     parser.add_argument(
-        "--apiid",
+        "--picpath",
         required=False,
-        help=" [?] Exhaustive research [work in progress] ...",
-    )
-    parser.add_argument(
-        "--apihash",
-        required=False,
-        help=" [?] Exhaustive research [work in progress] ...",
+        help="[?] where to store profile pics ...",
     )
     try:
         print_banner()
         args = parser.parse_args()
-        ton_ret = Ton_retriever(args.target, args.comprehensive, args.tor, args.silent)
-        ton_ret.start_searching()
-        if not args.silent:
-            ton_ret.print_info()
+
+        if not args.target and not args.login:
+            parser.print_help()
+            exit(0)
+
+        if args.login:
+            Ton_retriever.telegram_generate_session()
+            exit(0)
+        else:
+            ton_ret = Ton_retriever(
+                args.target,
+                args.comprehensive,
+                args.tor,
+                args.silent,
+                args.phonepivot,
+                args.picpath,
+            )
+            ton_ret.start_searching()
+            if not args.silent:
+                ton_ret.print_info()
 
     except KeyboardInterrupt:
         print("[-] ATOP was killed ...")
